@@ -5,15 +5,13 @@ from re import findall
 from re import sub as re_sub
 from shutil import move
 from sqlite3 import DatabaseError
-from typing import Dict
-from typing import List
 from typing import Optional
-from typing import Tuple
 from typing import Union
 
 from faapi import Journal
 from faapi import Submission
 from falocalrepo_database import FADatabaseTable
+from psutil import NoSuchProcess
 from psutil import Process
 from psutil import pids
 from requests import get as req_get
@@ -60,10 +58,13 @@ class Bar:
 
 
 def check_process(process: str) -> int:
-    return len(list(filter(
-        lambda p: "python" in p.name().lower() and any(process in split(cmd) for cmd in p.cmdline()),
-        map(Process, pids())
-    )))
+    ps: int = 0
+    for pid in pids():
+        try:
+            ps += "python" in (p := Process(pid)).name().lower() and any(process in split(cmd) for cmd in p.cmdline())
+        except NoSuchProcess:
+            pass
+    return ps
 
 
 def split_version(version: str, default=None, length=3) -> list:
@@ -136,7 +137,7 @@ def make_submission(id_: Union[int, str], author: str, title: str,
                     tags: str = "", description: str = "",
                     file_url: str = "", file_local_url: str = "",
                     folder: str = ""
-                    ) -> Tuple[Submission, Optional[bytes]]:
+                    ) -> tuple[Submission, Optional[bytes]]:
     id_ = int(id_)
     assert id_ > 0, "id must be greater than 0"
     assert isinstance(author, str) and author, "author must be of type str and not empty"
@@ -181,24 +182,30 @@ def make_submission(id_: Union[int, str], author: str, title: str,
     return sub, sub_file
 
 
-def search(table: FADatabaseTable, parameters: Dict[str, List[str]]) -> List[Dict[str, Union[int, str]]]:
-    parameters = {k.lower(): vs for k, vs in parameters.items()}
-    query: Dict[str, List[str]] = {k: vs for k, vs in parameters.items() if k not in ("order", "limit", "offset")}
-    if "author" in query:
-        query["replace(author, '_', '')"] = list(map(lambda u: clean_username(u, "%_"), query["author"]))
-        del query["author"]
-    if "username" in query:
-        query["username"] = list(map(lambda u: clean_username(u, "%_"), query["username"]))
-    return list(table.cursor_to_dict(table.select(
-        query,
-        like=True,
-        order=parameters.get("order", [table.column_id]),
-        limit=int(parameters.get("limit", 0)),
-        offset=int(parameters.get("offset", 0))
-    )))
+def search(table: FADatabaseTable, parameters: dict[str, list[str]], columns: list[str] = None
+           ) -> list[dict[str, Union[int, str]]]:
+    parameters = {k.upper(): vs for k, vs in parameters.items()}
+    query: dict[str, list[str]] = {k: vs for k, vs in parameters.items() if k in table.columns}
+    if "AUTHOR" in query:
+        query["REPLACE(AUTHOR, '_', '')"] = list(map(lambda u: clean_username(u, "%_"), query["AUTHOR"]))
+        del query["AUTHOR"]
+    if "USERNAME" in query:
+        query["USERNAME"] = list(map(lambda u: clean_username(u, "%_"), query["USERNAME"]))
+    if "ID" in query:
+        query["ID"] = list(map(lambda i: i.lstrip("0") if isinstance(i, str) else i, query["ID"]))
+    return list(table.cursor_to_dict(
+        table.select(
+            query,
+            columns=columns,
+            like=True,
+            order=parameters.get("order", [table.column_id]),
+            limit=int(parameters.get("limit", 0)),
+            offset=int(parameters.get("offset", 0))
+        ),
+        columns=columns))
 
 
-def print_items(items: List[Dict[str, Union[int, str]]]):
+def print_items(items: list[dict[str, Union[int, str]]]):
     space_term: int
 
     try:
@@ -220,20 +227,20 @@ def print_items(items: List[Dict[str, Union[int, str]]]):
         )
 
 
-def print_users(users: List[Dict[str, str]]):
-    space_folders: int = 7
-    space_folder: int = 9
+def print_users(users: list[dict[str, str]]):
     space_term: int = 10000
     try:
         space_term = get_terminal_size()[0]
     except IOError:
         pass
 
-    space_name_max: int = max([len(u["USERNAME"]) for u in users]) if users else 10
-    space_name: int = space_term - (space_folders + 3) - ((space_folder + 3) * 4) - 1
-    space_name = space_name_max if space_name > space_name_max else space_name
+    space_name: int = max([len(u["USERNAME"]) for u in users]) if users else 10
+    space_folders: int = max([len(u["FOLDERS"]) * 2 for u in users]) if users else 7
+    space_name = 8 if space_name < 8 else space_name
+    space_folders = 7 if space_folders < 7 else space_folders
+    space_name = space_name if (sn := space_term - space_folders - 3) > space_name else sn
 
     print(f"{'Username':^{space_name}} | {'Folders':^{space_folders}}")
     for user in sorted(users, key=lambda usr: usr["USERNAME"].lower()):
-        folders_min: str = " ".join(sorted(f[0] for f in user["FOLDERS"]))
+        folders_min: str = " ".join(sorted((f[:2] if f.startswith("!") else f[0]) for f in user["FOLDERS"]))
         print(f"{user['USERNAME'][:space_name]:<{space_name}} | {folders_min:^{space_folders}}")
