@@ -1,17 +1,15 @@
 from datetime import datetime
 from inspect import cleandoc
+from json import dumps
 from os import environ
 from os.path import abspath
 from os.path import getsize
 from os.path import isfile
 from os.path import join
 from re import match
+from sys import stderr
 from typing import Callable
-from typing import Dict
 from typing import Iterable
-from typing import List
-from typing import Set
-from typing import Tuple
 from typing import Union
 
 from faapi import __version__ as __faapi_version__
@@ -68,8 +66,8 @@ def docstring_parameter(*args, **kwargs):
     return inner
 
 
-def parameters_multi(args: Iterable[str]) -> Dict[str, List[str]]:
-    params: Dict[str, List[str]] = {}
+def parameters_multi(args: Iterable[str]) -> dict[str, list[str]]:
+    params: dict[str, list[str]] = {}
     for param, value in map(lambda p: p.split("=", 1), args):
         param = param.strip()
         params[param] = [*params.get(param, []), value]
@@ -77,13 +75,13 @@ def parameters_multi(args: Iterable[str]) -> Dict[str, List[str]]:
     return params
 
 
-def parameters(args: Iterable[str]) -> Dict[str, str]:
+def parameters(args: Iterable[str]) -> dict[str, str]:
     return {p: v for p, v in map(lambda p: p.split("=", 1), args)}
 
 
-def parse_args(args_raw: Iterable[str]) -> Tuple[Dict[str, str], List[str]]:
-    opts: List[str] = []
-    args: List[str] = []
+def parse_args(args_raw: Iterable[str]) -> tuple[dict[str, str], list[str]]:
+    opts: list[str] = []
+    args: list[str] = []
 
     for i, arg in enumerate(args_raw):
         if match(r"\w+=\w+", arg):
@@ -293,10 +291,10 @@ def download_users(db: FADatabase, *args: str):
     if len(args) != 2:
         raise MalformedCommand("users needs two arguments")
 
-    users_tmp: List[str] = list(filter(bool, map(clean_username, args[0].split(","))))
-    users: List[str] = sorted(set(users_tmp), key=users_tmp.index)
-    folders_tmp: List[str] = list(filter(bool, map(str.strip, args[1].split(","))))
-    folders: List[str] = sorted(set(folders_tmp), key=folders_tmp.index)
+    users_tmp: list[str] = list(filter(bool, map(clean_username, args[0].split(","))))
+    users: list[str] = sorted(set(users_tmp), key=users_tmp.index)
+    folders_tmp: list[str] = list(filter(bool, map(str.strip, args[1].split(","))))
+    folders: list[str] = sorted(set(folders_tmp), key=folders_tmp.index)
     download_users_(db, users, folders)
 
 
@@ -331,14 +329,14 @@ def download_update(db: FADatabase, *args: str):
         falocalrepo download update tom,jerry
     """
 
-    users: List[str] = []
-    folders: List[str] = []
+    users: list[str] = []
+    folders: list[str] = []
     opts, args = parse_args(args)
     if args and args[0] != "@":
-        users_tmp: List[str] = list(filter(bool, map(clean_username, args[0].split(","))))
+        users_tmp: list[str] = list(filter(bool, map(clean_username, args[0].split(","))))
         users = sorted(set(users_tmp), key=users_tmp.index)
     if args[1:] and args[1] != "@":
-        folders_tmp: List[str] = list(filter(bool, map(str.strip, args[1].split(","))))
+        folders_tmp: list[str] = list(filter(bool, map(str.strip, args[1].split(","))))
         folders = sorted(set(folders_tmp), key=folders_tmp.index)
     download_users_update(db, users, folders, int(opts.get("stop", 1)), opts.get("deactivated", "").lower() == "true")
 
@@ -361,8 +359,8 @@ def download_submissions(db: FADatabase, *args: str):
 
     if not args:
         raise MalformedCommand("submissions needs at least one argument")
-    sub_ids_tmp: List[str] = list(filter(str.isdigit, args))
-    sub_ids: List[str] = sorted(set(sub_ids_tmp), key=sub_ids_tmp.index)
+    sub_ids_tmp: list[str] = list(filter(str.isdigit, args))
+    sub_ids: list[str] = sorted(set(sub_ids_tmp), key=sub_ids_tmp.index)
     download_submissions_(db, sub_ids)
 
 
@@ -384,8 +382,8 @@ def download_journals(db: FADatabase, *args: str):
 
     if not args:
         raise MalformedCommand("journals needs at least one argument")
-    journal_ids_tmp: List[str] = list(filter(str.isdigit, args))
-    journal_ids: List[str] = sorted(set(journal_ids_tmp), key=journal_ids_tmp.index)
+    journal_ids_tmp: list[str] = list(filter(str.isdigit, args))
+    journal_ids: list[str] = sorted(set(journal_ids_tmp), key=journal_ids_tmp.index)
     download_journals_(db, journal_ids)
 
 
@@ -456,10 +454,12 @@ def database_history(db: FADatabase):
 def database_search_users(db: FADatabase, *args: str):
     """
     USAGE
-        falocalrepo database search-users [<param1>=<value1>] ...
-                    [<paramN>=<valueN>]
+        falocalrepo database search-users [json=<json>] [columns=<columns>]
+                    [<param1>=<value1>] ... [<paramN>=<valueN>]
 
     ARGUMENTS
+        <json>      Set to 'true' to output metadata in JSON format
+        <columns>   Comma-separated list of columns to select, only active for JSON
         <param>     Search parameter
         <value>     Value of the parameter
 
@@ -467,24 +467,35 @@ def database_search_users(db: FADatabase, *args: str):
         Search the users entries using metadata fields. Search parameters can be
         passed multiple times to act as OR values. All columns of the users table
         are supported. Parameters can be lowercase. If no parameters are supplied, a
-        list of all users will be returned instead.
+        list of all users will be returned instead. If <json> is set to 'true', the
+        results are printed as a list of objects in JSON format. If <columns> is
+        passed, then the objects printed with the JSON option will only contain
+        those fields.
 
     EXAMPLES
-        falocalrepo database search-users folders=%gallery% gallery=%0012345678%
+        falocalrepo database search-users json=true \\
+            folders=%gallery% gallery=%0012345678%
     """
 
-    results: List[Dict[str, str]] = search(db.users, parameters_multi(args))
-    print_users(results)
-    print(f"Found {len(results)} results")
+    opts = parameters_multi(args)
+    json, cols = opts.get("json", [None])[0] == "true", opts["columns"][0].split(",") if "columns" in opts else None
+    results: list[dict[str, Union[int, str]]] = search(db.users, opts, cols if cols and json else None)
+    if json:
+        print(dumps(results))
+    else:
+        print_users(results)
+        print(f"Found {len(results)} users")
 
 
 def database_search_submissions(db: FADatabase, *args: str):
     """
     USAGE
-        falocalrepo database search-submissions [<param1>=<value1>] ...
-                    [<paramN>=<valueN>]
+        falocalrepo database search-submissions [json=<json>] [columns=<columns>]
+                    [<param1>=<value1>] ... [<paramN>=<valueN>]
 
     ARGUMENTS
+        <json>      Set to 'true' to output metadata in JSON format
+        <columns>   Comma-separated list of columns to select, only active for JSON
         <param>     Search parameter
         <value>     Value of the parameter
 
@@ -492,27 +503,37 @@ def database_search_submissions(db: FADatabase, *args: str):
         Search the submissions entries using metadata fields. Search parameters can
         be passed multiple times to act as OR values. All columns of the submissions
         table are supported. Parameters can be lowercase. If no parameters are
-        supplied, a list of all submissions will be returned instead.
+        supplied, a list of all submissions will be returned instead. If <json> is
+        set to 'true', the results are printed as a list of objects in JSON format.
+        If <columns> is passed, then the objects printed with the JSON option will
+        only contain those fields.
 
     EXAMPLES
         falocalrepo database search-submissions tags=%|cat|%|mouse|% date=2020-% \\
             category=%artwork% order="AUTHOR" order="ID"
-        falocalrepo database search-submissions tags=%|cat|% tags=%|mouse|% \\
-            date=2020-% category=%artwork%
+        falocalrepo database search-submissions json=true columns=id,author,title \\
+            tags=%|cat|% tags=%|mouse|% date=2020-% category=%artwork%
     """
 
-    results: List[Dict[str, Union[int, str]]] = search(db.submissions, parameters_multi(args))
-    print_items(results)
-    print(f"Found {len(results)} results")
+    opts = parameters_multi(args)
+    json, cols = opts.get("json", [None])[0] == "true", opts["columns"][0].split(",") if "columns" in opts else None
+    results: list[dict[str, Union[int, str]]] = search(db.submissions, opts, cols if cols and json else None)
+    if json:
+        print(dumps(results))
+    else:
+        print_items(results)
+        print(f"Found {len(results)} results")
 
 
 def database_search_journals(db: FADatabase, *args: str):
     """
     USAGE
-        falocalrepo database search-journals [<param1>=<value1>] ...
-                    [<paramN>=<valueN>]
+        falocalrepo database search-journals [json=<json>] [columns=<columns>]
+                    [<param1>=<value1>] ... [<paramN>=<valueN>]
 
     ARGUMENTS
+        <json>      Set to 'true' to output metadata in JSON format
+        <columns>   Comma-separated list of columns to select, only active for JSON
         <param>     Search parameter
         <value>     Value of the parameter
 
@@ -520,18 +541,26 @@ def database_search_journals(db: FADatabase, *args: str):
         Search the journals entries using metadata fields. Search parameters can
         be passed multiple times to act as OR values. All columns of the journals
         table are supported. Parameters can be lowercase. If no parameters are
-        supplied, a list of all submissions will be returned instead.
+        supplied, a list of all submissions will be returned instead. If <json> is set
+        to 'true', the results are printed as a list of objects in JSON format. If
+        <columns> is passed, then the objects printed with the JSON option will only
+        contain those fields.
 
     EXAMPLES
         falocalrepo database search-journals date=2020-% author=CatArtist \\
             order="ID DESC"
-        falocalrepo database search-journals date=2020-% date=2019-% \\
-            content=%commission%
+        falocalrepo database search-journals json=true columns=id,author,title \\
+            date=2020-% date=2019-% content=%commission%
     """
 
-    results: List[Dict[str, Union[int, str]]] = search(db.journals, parameters_multi(args))
-    print_items(results)
-    print(f"Found {len(results)} results")
+    opts = parameters_multi(args)
+    json, cols = opts.get("json", [None])[0] == "true", opts["columns"][0].split(",") if "columns" in opts else None
+    results: list[dict[str, Union[int, str]]] = search(db.journals, opts, cols if cols and json else None)
+    if json:
+        print(dumps(results))
+    else:
+        print_items(results)
+        print(f"Found {len(results)} results")
 
 
 def database_add_user(db: FADatabase, *args):
@@ -555,15 +584,15 @@ def database_add_user(db: FADatabase, *args):
         falocalrepo database add-user username=tom folders=gallery,scraps
     """
 
-    make_params: Dict[str, str] = parameters(args)
+    make_params: dict[str, str] = parameters(args)
     username: str = db.users.new_user(make_params["username"])
     if make_params.get("folders", None) is not None:
-        folders: Set[str] = set(db.users[username]["FOLDERS"])
-        folders_new: Set[str] = set(filter(bool, map(str.lower, make_params["folders"].split(","))))
+        folders: set[str] = set(db.users[username]["FOLDERS"])
+        folders_new: set[str] = set(filter(bool, map(str.lower, make_params["folders"].split(","))))
         for f in folders - folders_new:
-            db.users.remove_user_folder(username, f) if f not in folders_new else None
+            db.users.remove_user_folder(username, f)
         for f in folders_new - folders:
-            db.users.add_user_folder(username, f) if f not in folders else None
+            db.users.add_user_folder(username, f)
     db.commit()
 
 
@@ -748,7 +777,7 @@ def database_merge(db: FADatabase, *args: str):
         raise FileNotFoundError(f"No such file or directory: '{args[0]}'")
     with FADatabase(args[0]) as db2:
         print(f"Merging with database {db2.database_path}...")
-        db.update(db2)
+        db.merge(db2)
         db.commit()
         print("Done")
 
@@ -913,13 +942,13 @@ def console(comm: str = "", *args: str) -> None:
         raise MultipleInstances(f"Another instance of {p} was detected")
 
     if environ.get("FALOCALREPO_DEBUG", None) is not None:
-        print(f"Using FALOCALREPO_DEBUG")
+        print(f"Using FALOCALREPO_DEBUG", file=stderr)
 
     # Initialise and prepare database
     database_path = "FA.db"
 
     if db_path := environ.get("FALOCALREPO_DATABASE", None):
-        print(f"Using FALOCALREPO_DATABASE: {db_path}")
+        print(f"Using FALOCALREPO_DATABASE: {db_path}", file=stderr)
         database_path = db_path if db_path.endswith(".db") else join(db_path, database_path)
 
     db: FADatabase = FADatabase(abspath(database_path))
