@@ -5,10 +5,7 @@ from os import get_terminal_size
 from time import sleep
 from typing import Any
 from typing import Callable
-from typing import Dict
-from typing import List
 from typing import Optional
-from typing import Tuple
 from typing import Union
 
 from faapi import DisabledAccount
@@ -30,7 +27,7 @@ class UnknownFolder(Exception):
     pass
 
 
-def read_cookies(db: FADatabase) -> List[Dict[str, str]]:
+def read_cookies(db: FADatabase) -> list[dict[str, str]]:
     return [{"name": n, "value": v} for n, v in json_loads(db.settings["COOKIES"]).items()]
 
 
@@ -50,7 +47,7 @@ def load_api(db: FADatabase) -> FAAPI:
     return api
 
 
-def download_items(db: FADatabase, item_ids: List[str], f: Callable[[FAAPI, FADatabase, int], Any]):
+def download_items(db: FADatabase, item_ids: list[str], f: Callable[[FAAPI, FADatabase, int], Any]):
     if item_ids_fail := list(filter(lambda i: not i.isdigit(), item_ids)):
         print("The following ID's are not correct:", *item_ids_fail)
     item_ids = list(filter(lambda i: i.isdigit(), item_ids))
@@ -60,12 +57,16 @@ def download_items(db: FADatabase, item_ids: List[str], f: Callable[[FAAPI, FADa
         f(api, db, item_id)
 
 
-def save_submission(db: FADatabase, sub: Submission, sub_file: Optional[bytes], user_update: bool = False):
-    db.submissions.save_submission({**dict(sub), "USERUPDATE": int(user_update)}, sub_file)
+def save_submission(db: FADatabase, sub: Submission, sub_file: Optional[bytes], sub_thumb: Optional[bytes],
+                    user_update: bool = False):
+    db.submissions.save_submission({**dict(sub), "USERUPDATE": int(user_update)}, sub_file, sub_thumb)
     db.commit()
 
 
 def download_submission_file(api: FAAPI, sub_file_url: str, *, speed: int = 100, bar: int = 10) -> Optional[bytes]:
+    if not sub_file_url:
+        return None
+
     bar: Bar = Bar(bar)
     file_binary: Optional[bytes] = bytes()
 
@@ -92,7 +93,7 @@ def download_submission_file(api: FAAPI, sub_file_url: str, *, speed: int = 100,
         bar.message("STOP")
         raise
     except (Exception, BaseException):
-        bar.message("ERR")
+        bar.message("ERROR")
         file_binary = None
     finally:
         bar.close("]")
@@ -105,16 +106,15 @@ def download_submission(api: FAAPI, db: FADatabase, submission: Union[int, Submi
     try:
         sub_id: int = submission.id if isinstance(submission, SubmissionPartial) else submission
         if sub_id in db.submissions:
-            Bar(length=10, message="IS IN DB").close()
+            Bar(length=10, message="IS IN DB").close("]")
             return True
         sub: Submission = api.get_submission(sub_id, False)[0]
-        if not sub.thumbnail_url and isinstance(submission, SubmissionPartial):
-            sub.thumbnail_url = submission.thumbnail_url
-        sub_file: Optional[bytes] = download_submission_file(api, sub.file_url, bar=7 if sub.thumbnail_url else 10)
-        if sub.thumbnail_url:
-            sub_thumbnail: Optional[bytes] = download_submission_file(api, sub.thumbnail_url, speed=0, bar=1)
-            db.submissions.save_submission_file(sub.id, sub_thumbnail, "thumbnail", "jpg", False)
-        save_submission(db, sub, sub_file, user_update)
+        if isinstance(submission, SubmissionPartial):
+            sub.thumbnail_url = sub.thumbnail_url or submission.thumbnail_url
+        save_submission(db, sub,
+                        download_submission_file(api, sub.file_url, bar=7 if sub.thumbnail_url else 10),
+                        download_submission_file(api, sub.thumbnail_url, speed=0, bar=1),
+                        user_update)
         return True
     except ParsingError:
         return False
@@ -122,7 +122,7 @@ def download_submission(api: FAAPI, db: FADatabase, submission: Union[int, Submi
         print()
 
 
-def download_submissions(db: FADatabase, sub_ids: List[str]):
+def download_submissions(db: FADatabase, sub_ids: list[str]):
     download_items(db, sub_ids, download_submission)
 
 
@@ -139,17 +139,18 @@ def download_journal(api: FAAPI, db: FADatabase, jrn_id: int):
     save_journal(db, journal)
 
 
-def download_journals(db: FADatabase, jrn_ids: List[str]):
+def download_journals(db: FADatabase, jrn_ids: list[str]):
     download_items(db, jrn_ids, download_journal)
 
 
-def download_users_update(db: FADatabase, users: List[str], folders: List[str], stop: int = 1, deactivated: bool = False):
+def download_users_update(db: FADatabase, users: list[str], folders: list[str], stop: int = 1,
+                          deactivated: bool = False):
     api: Optional[FAAPI] = None
     tot, fail = 0, 0
 
     users = list(map(clean_username, users))
     users = sorted(set(users), key=users.index)
-    users_db: List[dict] = sorted(
+    users_db: list[dict] = sorted(
         filter(lambda u: not users or u["USERNAME"] in users, db.users),
         key=lambda u: users.index(u["USERNAME"]) if users else u["USERNAME"])
 
@@ -168,7 +169,7 @@ def download_users_update(db: FADatabase, users: List[str], folders: List[str],
                 fail += fail_
         except UnknownFolder as err:
             print(f"Unknown folder: {err.args[0]}")
-            continue
+            raise
         except DisabledAccount:
             print(f"User {user} deactivated")
             db.users.deactivate_user(user)
@@ -183,7 +184,7 @@ def download_users_update(db: FADatabase, users: List[str], folders: List[str],
     print("Items failed:", fail) if fail else None
 
 
-def download_users(db: FADatabase, users: List[str], folders: List[str]):
+def download_users(db: FADatabase, users: list[str], folders: list[str]):
     api: Optional[FAAPI] = None
     users = list(map(clean_username, users))
     for user in sorted(set(users), key=users.index):
@@ -197,7 +198,7 @@ def download_users(db: FADatabase, users: List[str], folders: List[str]):
                 print("Items failed:", fail) if fail else None
         except UnknownFolder as err:
             print(f"Unknown folder: {err.args[0]}")
-            return
+            raise
         except DisabledAccount:
             print(f"User {user} disabled")
             db.users.deactivate_user(user)
@@ -211,7 +212,7 @@ def download_users(db: FADatabase, users: List[str], folders: List[str]):
             print(f"User {user} error: {repr(err)}")
 
 
-def download_user(api: FAAPI, db: FADatabase, user: str, folder: str, stop: int = 0) -> Tuple[int, int]:
+def download_user(api: FAAPI, db: FADatabase, user: str, folder: str, stop: int = 0) -> tuple[int, int]:
     items_total: int = 0
     items_failed: int = 0
     page: Union[int, str] = 1
@@ -223,7 +224,7 @@ def download_user(api: FAAPI, db: FADatabase, user: str, folder: str, stop: int
     found_items: int = 0
     skip: bool = False
 
-    download: Callable[[str, Union[str, int]], Tuple[List[Union[SubmissionPartial, Journal]], Union[int, str]]]
+    download: Callable[[str, Union[str, int]], tuple[list[Union[SubmissionPartial, Journal]], Union[int, str]]]
     exists: Callable[[int], Optional[dict]]
 
     if folder.startswith("!"):
